import math
from simpleai.search import SearchProblem,astar

#Tạo Map
MAP = """
##############################
#         #              #   #
# ####    ########       #   #
#  o #    #              #   #
#    ###     #####  ######   #
#      #   ###   #           #
#      #     #   #  #  #   ###
#     #####  ###    #  # x   #
#              #       #     #
##############################
"""
#Chuyển Map thành list

MAP=[list(x) for x in MAP.split("\n") if x]
#đinh giá để di chuyển xung quanh map. Di chuyển chéo thì sẽ cao hơn là di chuyển theo hướng thẳng.
cost_regular=1.0
cost_diagonal=1.7
#Gắn giá vào hướng di chuyển tương ứng
COST={
    "up":cost_regular,
    "down":cost_regular,
    "left":cost_regular,
    "right":cost_regular,
    "up left":cost_diagonal,
    "up right":cost_diagonal,
    "down left":cost_diagonal,
    "down right":cost_diagonal,
}
#Tạo Class MazeSolver thừa kế class SearchProblem
class MazeSolver(SearchProblem):
    def __init__(self,board):
        self.board=board
        self.goal=(0,0)
        #Tách những khối ban đầu và vị trí kết thúc:
        for y in range(len(self.board)):
            for x in range(len(self.board[y])):
                if self.board[y][x].lower()=="o":
                    self.initial=(x,y)
                elif self.board[y][x].lower()=="x":
                    self.goal=(x,y)
        super(MazeSolver,self).__init__(initial_state=self.initial)
    # Override method actions, tại mỗi vị trí chúng ta cần kiểm gia trá trị để đi đến ô kế tiếp và sau đó kết nối với các hành động(action) có thể. Nếu ô bên cạnh bị chặn thì hành động sẽ được bỏ qua
    def actions(self, state):
        actions=[]
        for action in COST.keys():
            newx,newy=self.result(state,action)
            if self.board[newy][newx]!="#":
                actions.append(action)
        return actions
    # Override method result. Dựa trên trạng thái hiện tại và hành động input, cập nhật vị trí tọa độ x và y
    def result(self, state, action):
        x,y=state
        if action.count("up"):
            y-=1
        if action.count("down"):
            y+=1
        if action.count("left"):
            x-=1
        if action.count("right"):
            x+=1
        new_state=(x,y)
        return new_state
    # Kiểm tra xem đã đến đích chưa ?
    def is_goal(self, state):
        return state==self.goal
    # Chúng ta định nghĩa hàm cost. đây là giá để di chuyển tới ô kế tiếp, và nó sẽ khác khi di chuyển ngang dọc và di chuyển chéo
    def cost(self,state,action,state2):
        return COST[action]
    # Giờ thì ta sử dụng hàm heuristic. Ở trường hợp này chúng ta sử dụng cách tính khoảng cách Euclidean ( tiên đề Euclidean):
    def heuristic(self, state):
        x,y=state
        gx,gy=self.goal
        return math.sqrt((x-gx)**2+(y-gy)**2)
    # Định nghĩa hàm main
if __name__=="__main__":
    problem=MazeSolver(MAP)
    result=astar(problem,graph_search=True)
    # Xem path kết quả


    path=[x[1] for x in result.path()]
    # In kết quả ra Terminal
    print()
    for y in range(len(MAP)):
        for x in range(len(MAP[y])):
            if(x,y)==problem.initial:
                print('0',end='')
            elif(x,y)==problem.goal:
                print('x',end='')
            elif(x,y) in path:
                print('.',end='')
            else:
                print(MAP[y][x],end='')
        print()

